import cv2
import numpy as np
from tensorflow.keras.models import load_model # type: ignore
import os

# Access environment variables
DEBUGGING = False

def extract_patches(image):
    height, width, _ = image.shape
    square_height = height // 3
    square_width = width // 3
    patches = []
    for row in range(3):
        for col in range(3):
            y_start = row * square_height
            y_end = (row + 1) * square_height
            x_start = col * square_width
            x_end = (col + 1) * square_width
            patch = image[y_start:y_end, x_start:x_end]
            patches.append(patch)
    return patches

def preprocess_patches(patches):
    patches_resized = [cv2.resize(patch, (32, 32)) for patch in patches]
    patches_normalized = np.array(patches_resized) / 255.0
    return patches_normalized

def predict_colors(model, image):
    patches = extract_patches(image)
    patches_preprocessed = preprocess_patches(patches)

    predictions = model.predict(patches_preprocessed)
    predicted_colors = [np.argmax(pred) for pred in predictions]
    return predicted_colors

def indices_to_colors(indices):

    color_map = {0: 'white', 1: 'yellow', 2: 'red', 3: 'blue', 4: 'green', 5: 'orange'}
    return [color_map[idx] for idx in indices]


def main():
    model_path = "rubiks_cube_model.h5"
    if not os.path.exists(model_path):
        print("Error: Model file not found.")
        return
    print("Loading model...")
    model = load_model(model_path)

    # Initialize OpenCV camera
    camera = cv2.VideoCapture(0)  # Use the default camera (index 0)
    if not camera.isOpened():
        print("Error: Could not open the camera.")
        return

    while True:
        # Capture a frame
        ret, frame = camera.read()
        if not ret:
            print("Error: Failed to capture an image.")
            continue

        # Display the live camera feed
        cv2.imshow("Rubik's Cube - Press 'c' to capture", frame)

        # Wait for the user to press 'c' to capture or 'q' to quit
        key = cv2.waitKey(1) & 0xFF
        if key == ord('c'):
            # Show the captured image to the user
            while True:
                cv2.imshow("Captured Image - Press 'y' to proceed or 'r' to retake", frame)
                key_confirm = cv2.waitKey(0) & 0xFF
                if key_confirm == ord('y'):
                    print("Processing captured image...")

                    predicted_colors = predict_colors(model, frame)
                    colors = None
                    if(DEBUGGING):
                        colors = indices_to_colors(predicted_colors)
                        print(colors)
                    else:
                        print("Predicted Colors:", predicted_colors)
                    break  # Exit confirmation loop and return to live feed
                elif key_confirm == ord('r'):
                    print("Retaking the picture...")
                    break  # Retake the picture

        elif key == ord('q'):
            print("Exiting...")
            break

    camera.release()
    cv2.destroyAllWindows()

if __name__ == "__main__":
    main()


#     label_map = {'white': 0, 'yellow': 1, 'red': 2, 'blue': 3, 'green': 4, 'orange': 5}
