import cv2
import numpy as np
from tensorflow.keras.models import Sequential # type: ignore
from tensorflow.keras.layers import Conv2D, MaxPooling2D, Flatten, Dense # type: ignore
import os
import json

DEBUGGING = False
#   extract 9 patches from the image assuming the image
#    contains only the cube and is properly aligned.
def extract_patches(image):

    height, width, _ = image.shape
    square_height = height // 3
    square_width = width // 3
    patches = []
    for row in range(3):
        for col in range(3):
            y_start = row * square_height
            y_end = (row + 1) * square_height
            x_start = col * square_width
            x_end = (col + 1) * square_width
            patch = image[y_start:y_end, x_start:x_end]
            patches.append(patch)
    return patches

# preprocess patches (resize to 32x32 and normalize)
def preprocess_patches(patches):
    patches_resized = [cv2.resize(patch, (32, 32)) for patch in patches]
    patches_normalized = np.array(patches_resized) / 255.0  # Normalize to range [0, 1]
    return patches_normalized

# CNN model
def build_model():

    model = Sequential([
        Conv2D(32, (3, 3), activation='relu', input_shape=(32, 32, 3)),
        MaxPooling2D((2, 2)),
        Flatten(),
        Dense(64, activation='relu'),
        Dense(6, activation='softmax')  # 6 classes for Rubik's cube colors
    ])
    model.compile(optimizer='adam', loss='sparse_categorical_crossentropy', metrics=['accuracy'])
    return model

#load training data
def load_training_data(data_dir):

    images = []
    labels = []
    label_map = {'white': 0, 'yellow': 1, 'red': 2, 'blue': 3, 'green': 4, 'orange': 5}
    for label_name, label_idx in label_map.items():
        folder_path = os.path.join(data_dir, label_name)
        if not os.path.exists(folder_path):
            continue
        for file_name in os.listdir(folder_path):
            file_path = os.path.join(folder_path, file_name)
            image = cv2.imread(file_path)
            if image is not None:
                image_resized = cv2.resize(image, (32, 32))
                images.append(image_resized)
                labels.append(label_idx)
    images = np.array(images) / 255.0  # Normalize
    labels = np.array(labels)
    return images, labels

def load_test_labels(label_file):
    with open(label_file, 'r') as f:
        labels = {}
        for line in f:
            image_name, label_str = line.strip().split(":")
            labels[image_name.strip()] = json.loads(label_str.strip())
    return labels

def main():
    script_dir = os.path.dirname(os.path.abspath(__file__))

    # Paths
    training_data_dir = os.path.join(script_dir, "training_data")
    test_images_dir = os.path.join(script_dir, "test_data")
    test_labels_file = os.path.join(test_images_dir, "test_labels.txt")

    print("Loading training data...")
    X_train, y_train = load_training_data(training_data_dir)
    print(f"Loaded {len(X_train)} training samples.")

    print("Building model...")
    model = build_model()

    print("Training model...")
    model.fit(X_train, y_train, epochs=20, batch_size=16, validation_split=0.2)

    model.save("rubiks_cube_model.h5")
    print("Model saved as rubiks_cube_model.h5.")

    print("Loading test labels...")
    test_labels = load_test_labels(test_labels_file)

    print("Processing test images...")
    correct = 0
    total = 0

    for file_name in os.listdir(test_images_dir):
        if file_name.endswith(".jpg"):
            file_path = os.path.join(test_images_dir, file_name)
            test_image = cv2.imread(file_path)
            test_patches = extract_patches(test_image)
            test_patches_preprocessed = preprocess_patches(test_patches)

            predictions = model.predict(test_patches_preprocessed)
            predicted_colors = [np.argmax(pred) for pred in predictions]

            if file_name in test_labels:
                true_colors = test_labels[file_name]
                total += 9
                for i, (pred, true) in enumerate(zip(predicted_colors, true_colors)):
                    if pred == true:
                        correct += 1
                    else:
                        # Show the incorrect patch
                        patch = test_patches[i]
                        if(DEBUGGING):
                            cv2.imshow("Incorrect Patch", patch)
                            print(f"File: {file_name}, Patch: {i}")
                            print(f"Predicted: {pred}, True: {true}")
                            print("Press any key to continue...")
                            cv2.waitKey(0)  # Wait for a key press
                            cv2.destroyAllWindows()  # Close the window

    accuracy = (correct / total) * 100 if total > 0 else 0
    print(f"Accuracy: {accuracy:.2f}%")

if __name__ == "__main__":
    main()





    # label_map = {'white': 0, 'yellow': 1, 'red': 2, 'blue': 3, 'green': 4, 'orange': 5}
